;***************************************;***************************************
;
;    Filename:         					Main.asm
;    Date:             					11/12/2012
;    File Version:                      V0.5
;    Author:                            jeff bachiochi
;    Company:                           imagine that
;										www.imaginethatnow.com
;
;***************************************;***************************************
;	Version
;
;	0.1 used PWM for a non oscillating piezo
;	0.2	uses oscillating piezo to eliminate wasting one timer for PWM
;	0.3 uses Timer 1 in timer mode, not capture mode
;	0.4 uses Timer 0 & 1 in timer mode
;	0.5 added LCD
;
;***************************************;***************************************
;
 	.cdecls	C,LIST,"msp430g2553.h"
Constants	.text
;***** PORT1
LED1	.equ	0						; LED (sounder ON/OFF)
RX1		.equ	1						; USCI receiver
TX1		.equ	2						; USCI transmitter
SW2		.equ	3						; Push Button Mode
D4		.equ	4						; LCD Data
D5		.equ	5						; LCD Data
D6		.equ	6						; LCD Data
D7		.equ	7						; LCD Data
;
;***** PORT2
Mike	.equ	0						; Mike Input (tone ON/OFF)
E		.equ	1						; LCD Strobe
RS		.equ	2						; LCD Register Select
R_W		.equ	3						; LCD Read Write
Key		.equ	4						; External Keyer
DitCLK	.equ	5						;
;		.equ	6						;
;		.equ	7						;
;
;***** Flags
TXBufferEmpty	.equ	0				; 0=not empty, 1=empty
TXBufferFull	.equ	1				; 0=not full, 1=full
RXBufferEmpty	.equ	2				; 0=not empty, 1=empty
RXBufferFull	.equ	3				; 0=not full, 1=full
Tock	.equ	4						; 1=timeup 0=timing
Sound	.equ	5						; 1=InProcess, 0=Finished
NewON	.equ	6						; 1=new ON Time available
NewOFF	.equ	7						; 1=new OFF Time available
T1OVF	.equ	8						; 1=occurred
Letter	.equ	9						; 1=InProcess, 0=Finished
Space	.equ	10						; 1=space character
Pause	.equ	11						; 1=InProcess, 0=Finished
Clear	.equ	12						; 1=not all spaces
;
OSC		.equ	8000000
BAUD	.equ	9600
BAUDCLOCKH	.equ	OSC/BAUD/0x100
BAUDCLOCKL	.equ	OSC/BAUD-(BAUDCLOCKH*0x100)
BufferLength	.equ	80h
MBufferLength	.equ	10
LCDBufferLength	.equ	40h
DITTime	.equ	140						; x*1.6mS
ShiftCountReloadValue	.equ	0
Timer1TimeOutReloadValue	.equ	500
;
;---------------------------------------;---------------------------------------
;
MyVars	.data
Timer1TimeOutReload	.word	0
TX_Head		.word	0
TX_Tail		.word	0
RX_Head		.word	0
RX_Tail		.word	0
M_Head		.word	0
Flags		.word	0
ONTime		.word	0
OFFTime		.word	0
TDIT 		.word	0
RDIT 		.word	0
TableAddress	.word	0
Line1Pointer	.word	0
Line2Pointer	.word	0
ShiftCountReload	.word	0
ShiftCount	.word	0
RXCharacterIn	.byte	0
RXCharacterOut	.byte	0
TXCharacterIn	.byte	0
TXCharacterOut	.byte	0
LCDCharacterIn	.byte	0
SymbolCount	.byte	0
DataIn		.byte	0
DataOut		.byte	0
EndOfMyVars	.byte	0
;
			.bss TXBuffer, 	BufferLength	; 80h characters
			.bss RXBuffer, 	BufferLength	; 80h characters
			.bss LCDLine1,	LCDBufferLength		; 40h characters
			.bss LCDLine2,	LCDBufferLength		; 40h characters
			.bss MORSEBuffer, 	MBufferLength	; 10 characters
;
;---------------------------------------;---------------------------------------
;
;	main program loop
;
;	R9	-	message ponter
;
;	call	-	Init (R4)
;	call	-	SendMessage (R4, R5, R9)
;	call	-	CheckRXBuffer (R4, R5)
;	call	-	CheckMorse (R4, R5, R6)
;	call	-	SendNumber (R4, R5, R6, R7, R8, R9)
;
;	int	-	TX_ISR (R10, R11)
;	int	-	RX_ISR (R10, R11)
;	int	-	TimerA0_CCI_ISR
;	int	-	TimerA1_CCI_ISR
;
;---------------------------------------;---------------------------------------
;
            .text                       ; Progam Start
            .global _main
_main
			mov.w   #03FFh,SP           ; Initialize stackpointer
;
StopWDT     mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
			call 	#Init
;
			mov		&TDIT,		&TA0CCR0	; 1 DIT Tick (to allow DCO to stabilize)
			bis		#TACLR,		&TA0CTL	; zero's Timer 0
			bis		#MC0,		&TA0CTL	; begin Timer0
			bis		#1<<Pause,	&Flags	; now Pause In Process
PauseLoop
;
			bit		#1<<Pause,	&Flags	; Pause In Process?
			jnz		PauseLoop			; yes, wait
;
InitializeLCD
			call	#LCD_Init
;
SendUARTMessage
			mov		#Message_SIGNON1,	R9
			call	#SendMessage
;
			call	#SendCRLF

			mov		#Message_SIGNON2,	R9
			call	#SendMessage
;
			call	#SendCRLF
;
SendUARTMessageWaitUntilSent
			mov		TX_Head,	R4
			mov		TX_Tail,	R5
			cmp		R4,			R5
			jne		SendUARTMessageWaitUntilSent
;
SendLCDMessage
			mov		#Message_Morse,	R9
			call	#SendToLCDLine1Buffer
;
			call	#SendToLCDLine1
;
			mov		#Message_ftb272,	R9
			call	#SendToLCDLine2Buffer
;
			call	#SendToLCDLine2
;
;
;---------------------------------------;---------------------------------------
;
;	Main Loop
;
;---------------------------------------;---------------------------------------
;
MainLoop
;---------------------------------------
MainLoop_SendMorse
			bit		#1<<Sound,	&Flags	; InProcess?
			jnz		MainLoop_SendMorse_Exit	; Yes, exit
;
			bit		#1<<Letter,	&Flags	; InProcess?
			jnz		MainLoop_SendMorse_DoLetter	; Yes, go on
;
			call	#CheckRXBuffer
;
			bit		#1<<RXBufferEmpty,	&Flags	; Is RXBufferEmpty (z=0 not empty)
			jnz		MainLoop_SendMorse_Exit		; jump if RXBuffer is Empty
;
MainLoop_SendMorse_DoCharacter
			mov		#20h,	R4
			cmp.b	&RXCharacterOut,	R4
			jz		MainLoop_SendMorse_Space
;
MainLoop_SendMorse_ChkL30h
			mov		#30h,	R4
			cmp.b	R4,		&RXCharacterOut
			jl		MainLoop_SendMorse_Exit
;
MainLoop_SendMorse_ChkL3Ah
			mov		#3Ah,	R4
			cmp.b	R4,		&RXCharacterOut
			jl		MainLoop_SendMorse_Numeric
;
MainLoop_SendMorse_ChkL41h
			mov		#41h,	R4
			cmp.b	R4,		&RXCharacterOut
			jl		MainLoop_SendMorse_Exit
;
MainLoop_SendMorse_ChkL5Bh
			mov		#5Bh,	R4
			cmp.b	R4,		&RXCharacterOut
			jl		MainLoop_SendMorse_Alpha
;
MainLoop_SendMorse_ChkL61h
			mov		#61h,	R4
			cmp.b	R4,		&RXCharacterOut
			jl		MainLoop_SendMorse_Exit
;
MainLoop_SendMorse_ChkL7Bh
			mov		#7Bh,	R4
			cmp.b	R4,		&RXCharacterOut
			jl		MainLoop_SendMorse_Alpha
;
			jmp		MainLoop_SendMorse_Exit
;
MainLoop_SendMorse_Numeric
			mov.b	&RXCharacterOut,	R4	; get numeric
			mov		#22,	R5			; get offset
			sub		R5,		R4			; new pointer
			rla		R4					; alignment
			rla		R4					; alignment
			rla		R4					; alignment
			mov		#Table,	R5
			add		R5,		R4
			mov		R4,		&TableAddress
			bis		#1<<Letter,		&Flags	; we got a MORSE letter to send
			jmp		MainLoop_SendMorse
;
MainLoop_SendMorse_Alpha
			clr		R4
			mov.b	&RXCharacterOut,	R4	; get alpha
			mov		#0x41,		R5		; get offset
			sub		R5,		R4			; new pointer
			bic		#00100000b,	R4		; force lower case to upper
			rla		R4					; alignment
			rla		R4					; alignment
			rla		R4					; alignment
			mov		#Table,	R5
			add		R5,		R4
			mov		R4,		&TableAddress
			bis		#1<<Letter,		&Flags	; we got a MORSE letter to send
			jmp		MainLoop_SendMorse
;
MainLoop_SendMorse_Space
			mov		&TDIT,		R6		; 1 DITTime
			rla		R6					; 2 DITTime
			rla		R6					; 4 DITTime
			mov		R6,			&TA0CCR0	; do a space, 4 DIT Times+3 as end-of-letter
			bis		#1<<Space,		&Flags	; we got a space to send
			bis		#1<<Letter,		&Flags	; we got a MORSE letter to send
			bis		#1<<Sound,	&Flags	; (but no sound)
			bis		#TACLR,		&TA0CTL	; zero's Timer 0
			bis		#MC0,		&TA0CTL	; begin Timer 0
			jmp		MainLoop_SendMorse_Exit
;
MainLoop_SendMorse_DoCharacter_Exit
 			jmp		MainLoop_SendMorse_Exit
;
MainLoop_SendMorse_DoLetter
			bit		#1<<Space,		&Flags
			jnz		MainLoop_SendMorse_DoSpace_Done
;
			mov		&TableAddress,	R6
			mov.b	@R6,	R5			; get table entry
			tst.b	R5					; is it zero?
			jz		MainLoop_SendMorse_DoLetter_Done	; yes, done
;
			mov.b	#".",	R6			; no, compare
			cmp.b	R6,		R5			; is it a '1'
			jz		MainLoop_SendMorse_DoLetter_1	; yes, do a DIT
;
MainLoop_SendMorse_DoLetter_3
			mov		&TDIT,		R6		; 1 DITTime
			add		&TDIT,		R6		; 2 DITTime
			add		&TDIT,		R6		; 3 DITTime
			mov		R6,			&TA0CCR0	; do a DAH, 3 DITTimes
			jmp		MainLoop_SendMorse_DoLetter_Exit
;
MainLoop_SendMorse_DoLetter_1
			mov		&TDIT,		TA0CCR0		; do a DIT, 1 DITTime
			jmp		MainLoop_SendMorse_DoLetter_Exit
;
MainLoop_SendMorse_DoSpace_Done
			bic		#1<<Space,	&Flags
;
MainLoop_SendMorse_DoLetter_Done
			bic		#1<<Letter,	&Flags
			mov		&TDIT,		R6		; 1 DITTime
			mov		R6,			&TA0CCR0	; do 1 additional DITTimes, leave output OFF
			bis		#1<<Sound,	&Flags
			bis		#TACLR,		&TA0CTL	; zero's Timer 0
			bis		#MC0,		&TA0CTL	; begin Timer0
			jmp		MainLoop_SendMorse_Exit
;
MainLoop_SendMorse_DoLetter_Exit
			bis		#1<<LED1,	&P1OUT
			bis		#1<<Sound,	&Flags
			bis		#TACLR,		&TA0CTL	; zero's Timer 0
			bis		#MC0,		&TA0CTL	; begin Timer 0
			inc		&TableAddress
 			jmp		MainLoop_SendMorse_Exit
;
MainLoop_SendMorse_Exit
;---------------------------------------
MainLoop_ManualKey
			bit		#1<<Key,	&P2IN
			jnz		MainLoop_ManualKey_Exit
;
			bis		#1<<LED1,	&P1OUT	; enable external sounder
;
MainLoop_ManualKey_Pressed
			call	#Delay
;
			bit		#1<<Key,	&P2IN
			jnz		MainLoop_ManualKey_NotPressed	; P2IN.Key=1 (not pressed)
;
			jmp		MainLoop_ManualKey_Pressed
;
MainLoop_ManualKey_NotPressed
			call	#Delay
;
			bit		#1<<Key,	&P2IN
			jz		MainLoop_ManualKey_Pressed	; P2IN.Key=0 (pressed)
;
			bic		#1<<LED1,	&P1OUT	; disable external sounder
			jmp		MainLoop_ManualKey_Exit
;
MainLoop_ManualKey_Exit
;
;---------------------------------------
;
MainLoop_Exit
            jmp		MainLoop            ;
;
;---------------------------------------
;
Delay
			dec		&ShiftCount
			jnz		Delay
;
			mov		&ShiftCountReload,	&ShiftCount
			ret
;
;---------------------------------------;---------------------------------------
;
;	Initialize registers
;
;---------------------------------------;---------------------------------------
;
Init
;
SetupP1
            mov.b   #00001100b,	&P1OUT 	; D7, D6, D5, D4, SW2, TX1=1, RX1, LED1=0
     		mov.b   #00000101b,	&P1DIR  ; D7, D6, D5, D4, C=I, SW2=I, TX1=O, RX1=I, LED1=O
            bis.b   #00001000b,	&P1REN  ; D7, D6, D5, D4, SW2=E, TX1, RX1, LED1
            bis.b   #00000000b,	&P1IE   ; D7, D6, D5, D4, SW2, TX1, RX1, LED1
            bis.b   #00000000b,	&P1IES 	; D7, D6, D5, D4, SW2, TX1, RX1, LED1
            bic.b   #00000000b,	&P1IFG  ; D7, D6, D5, D4, SW2, TX1, RX1, LED1
            bis.b   #00000110b,	&P1SEL ; D7, D6, D5, D4, SW2, TX1=A, RX1=A, LED1
            bis.b   #00000110b,	&P1SEL2 ; D7, D6, D5, D4, SW2, TX1, RX1, LED1
;
SetupP2     mov.b   #00101110b,	&P2DIR  ; x, x, DitCLK=O, Key=I, R_W=O, RS=O, E=O, Mike=I
            mov.b   #00010000b,	&P2OUT 	; x, x, DitCLK, Key, R_W, RS, E, Mike
            bis.b   #00010001b,	&P2REN  ; x, x, DitCLK, Key=E, R_W, RS, E, Mike=E
            bis.b   #00000000b,	&P2IE   ; x, x, DitCLK, Key, R_W, RS, E, Mike
            bis.b   #00000001b,	&P2IE   ; x, x, DitCLK, Key, R_W, RS, E, Mike=E
            bis.b   #00000000b,	&P2IES 	; x, x, DitCLK, Key, R_W, RS, E, Mike
            bic.b   #00000000b,	&P2IFG  ; x, x, DitCLK, Key, R_W, RS, E, Mike
            bis.b   #00000000b,	&P2SEL 	; x, x, DitCLK, Key, R_W, RS, E, Mike
            bis.b   #00000000b,	&P2SEL2 ; x, x, DitCLK, x, Key, R_W, RS, E, Mike
;
SetupDCO								; MCLK=8MHz
			mov.b 	&CALDCO_8MHZ,	&DCOCTL 	; Set DCO step + modulation
			mov.b	&CALBC1_8MHZ,	&BCSCTL1 	; Set range
			mov.b	#10001101b,	&BCSCTL1	; XT2OFF=1, XTS=0, DIVA5:4=00, RSEL3:0=1101 (ACLK~12kHz)
			mov.b	#00000000b,	&BCSCTL2	; SELM7:6=00, DIVM5:4=00, SELS=0, DIVS2:1=00, DCOR=0
			mov.b	#00000000b,	&BCSCTL3	; XT2S7:6=00, LFXT1S5:4=00, XCAP3:2=00, XT2OF=0, LFXT1OF=0
;
SetupUSCI
			bic.b	#UCSWRST,	&UCA0CTL1	; enable USCI
			bis.b	#UCSSEL1,	&UCA0CTL1	; SMCLK
			bis.b	#UCSPB,		&UCA0CTL0	; 2 stop bits
			mov.b	#BAUDCLOCKL,	&UCA0BR0	; baudrate divider low byte
			mov.b	#BAUDCLOCKH,	&UCA0BR1	; baudrate divider high byte
			mov		#00000100b,	&UCA0MCTL	; UCBRF7:4=0000, UCBRS3:1=010, UCOS16=0
            bic.b   #UCA0RXIFG,	&IFG2  	; clear interrupts
            							; x, x, x, x, x, x, UCA0TXIE=0 UCA0RXIE=1
            bis.b   #UCA0RXIE,	&IE2    ; enable interrupts
            							; x, x, x, x, x, x, UCA0TXIE=0 UCA0RXIE=1
;
SetupTimerA0
			mov		#0000000111000010b,	&TA0CTL	; x, x, x, x, x, x, TASSEL9:8=01,
										; ID7:6=11, MC5:4=00 x, TACLR=0, TAIE=1, TAIFG=0
										; ACK
			mov		#0000000000000000b,	&TA0CCTL0	; CM15:14=00, CCIS13:12=00, SCS=0, SCCI=0, x, CAP=0,
										; OUTMOD7:5=000, CCIE=0, CCI=0, OUT=0, COV=0, CCIFG=0
;
SetupTimerA1
			mov		#0000000111000010b,	&TA1CTL	; x, x, x, x, x, x, TASSEL9:8=01,
										; ID7:6=11, MC5:4=00 x, TACLR=0, TAIE=1, TAIFG=0
										; ACLK
			mov		#0000000000000000b,	&TA1CCTL0	; CM15:14=00, CCIS13:12=00, SCS=0, SCCI=0, x, CAP=0,
										; OUTMOD7:5=000, CCIE=0, CCI=0, OUT=0, COV=0, CCIFG=0
			mov		#Timer1TimeOutReloadValue,	&TA1CCR0	; 500=~0.5 Sec
;
SetupVars
			mov		#TXBuffer,	R4
			mov		#EndOfMyVars,	R5
;
SetupVars_Loop
			mov		#0,			0(R4)
			inc		R4
			cmp		R4,			R5
			jnz		SetupVars_Loop
;
			mov		#TXBuffer,	&TX_Head
			mov		#TXBuffer,	&TX_Tail
			mov		#RXBuffer,	&RX_Head
			mov		#RXBuffer,	&RX_Tail
			mov		&RX_Head,	R4
			mov		#Message_SIGNON1,	R9
;
SendMorseLoop
			mov.b	@R9,	R5
			tst.b	R5					; is it zero?
			jz		SendMorse_Exit	; yes, done
;
			mov.b	R5,		0(R4)
			inc		R9
			inc		R4
			jmp		SendMorseLoop
;
SendMorse_Exit
			mov		R4,		&RX_Head
;
			mov		#DITTime,	&TDIT
			mov		#DITTime,	&RDIT
			mov		#MORSEBuffer,	&M_Head
			mov		#LCDLine1,	&Line1Pointer
			mov		#LCDLine2,	&Line2Pointer
			mov		#ShiftCountReloadValue,	&ShiftCountReload
			mov		#Timer1TimeOutReloadValue,	&Timer1TimeOutReload
;
			eint
;
            ret
;
;---------------------------------------;---------------------------------------
;   Send a text message to UART
;
;	TXCharacterIn	-	character to add to buffer
;	R9	-	message pointer
;
;	call	-	AddToTXBuffer (R4, R5)
;
;---------------------------------------;---------------------------------------
;
SendMessage
			mov.b	@R9,	&TXCharacterIn
			tst.b	&TXCharacterIn		; is it zero?
			jz		SendMessage_Exit	; yes, done
;
			call	#AddToTXBuffer
			inc		R9
			jmp		SendMessage
;
SendMessage_Exit
			ret
;
;---------------------------------------;---------------------------------------
;   Send CR LF to UART
;
;	R9	-	message pointer
;
;	call	-	AddToTXBuffer (R4, R5)
;
;---------------------------------------;---------------------------------------
;
SendCRLF
			mov.b	#0Dh,	&TXCharacterIn	; CR
			call	#AddToTXBuffer
;
			mov.b	#0Ah,	&TXCharacterIn	; LF
			call	#AddToTXBuffer
;
			ret
;
;---------------------------------------;---------------------------------------
;   Send a text message to LCD Line1
;
;	R9	-	message pointer
;	R8	-	LCD pointer
;
;---------------------------------------;---------------------------------------
;
SendToLCDLine1
			mov.b	#080h,	&DataOut
			call	#LCD_W_Control
;
			mov.b	#8,		R8
			mov		&Line1Pointer,	R9
			bic		#1<<Clear,		&Flags
;
SendToLCDLine1_Loop
			mov.b	@R9,	&DataOut
			mov.b	&DataOut,	R4
			sub.b	#020h,		R4
			jz		SendToLCDLine1_Loop1
;
			bis		#1<<Clear,		&Flags
;
SendToLCDLine1_Loop1
			call	#LCD_W_Data
;
			inc		R9
			dec.b	R8
			jnz		SendToLCDLine1_Loop
;
			bit		#1<<Clear,		&Flags
			jnz		SendToLCDLine1_Exit
;
			mov		#LCDLine1,		&Line1Pointer
;
SendToLCDLine1_Exit
			ret
;
;---------------------------------------;---------------------------------------
;   Send a text message to LCD Line2
;
;	R9	-	message pointer
;	R8	-	LCD pointer
;
;---------------------------------------;---------------------------------------
;
SendToLCDLine2
			mov.b	#0C0h,	&DataOut
			call	#LCD_W_Control
;
			mov.b	#8,		R8
			mov		&Line2Pointer,	R9
			bic		#1<<Clear,		&Flags
;
SendToLCDLine2_Loop
			mov.b	@R9,	&DataOut
			mov.b	&DataOut,	R4
			sub.b	#020h,		R4
			jz		SendToLCDLine2_Loop1
;
			bis		#1<<Clear,		&Flags
;
SendToLCDLine2_Loop1
			call	#LCD_W_Data
;
			inc		R9
			dec.b	R8
			jnz		SendToLCDLine2_Loop
;
			bit		#1<<Clear,		&Flags
			jnz		SendToLCDLine2_Exit
;
			mov		#LCDLine2,		&Line2Pointer
;
SendToLCDLine2_Exit
			ret
;
;---------------------------------------;---------------------------------------
;   Send a text message to LCD Line1 Buffer
;
;	R9	-	message pointer
;	R4	-	LCD pointer
;
;---------------------------------------;---------------------------------------
;
SendToLCDLine1Buffer
			mov		#LCDLine1,	R4
;
SendToLCDLine1Buffer_Loop
			mov.b	@R9,	&DataOut
			tst.b	&DataOut			; is it zero?
			jz		SendToLCDLine1Buffer_Exit	; yes, done
;
			mov.b	&DataOut,	0(R4)
			inc		R9
			inc		R4
			jmp		SendToLCDLine1Buffer_Loop
;
SendToLCDLine1Buffer_Exit
			ret
;
;---------------------------------------;---------------------------------------
;   Send a text message to LCD Line2 Buffer
;
;	R9	-	message pointer
;	R4	-	LCD pointer
;
;---------------------------------------;---------------------------------------
;
SendToLCDLine2Buffer
			mov		#LCDLine2,	R4
;
SendToLCDLine2Buffer_Loop
			mov.b	@R9,	&DataOut
			tst.b	&DataOut			; is it zero?
			jz		SendToLCDLine2Buffer_Exit	; yes, done
			mov.b	&DataOut,	0(R4)
			inc		R9
			inc		R4
			jmp		SendToLCDLine2Buffer_Loop
;
SendToLCDLine2Buffer_Exit
			ret
;
;---------------------------------------;---------------------------------------
;     Check the RXBuffer for Characters
;
;	R4	-	temp
;	R5	-	temp
;
;---------------------------------------;---------------------------------------
;                		            ;
CheckRXBuffer
			dint
			bis		#1<<RXBufferEmpty,	Flags	; start empty
			mov		&RX_Head,	R5
			mov		&RX_Tail,	R4		;
			cmp		R5,			R4		; R4=R5 (z=1 same)
			jz		CheckRXBuffer_Exit    	; jump if same
;
CheckRXBuffer_BufferNotEmpty
			bic		#1<<RXBufferEmpty,	Flags	; not empty
			mov.b	@R4,		&RXCharacterOut
			inc		R4
			mov		#RXBuffer+BufferLength,	R5	; R5=Buffer Overflow
			cmp		R4,			R5		; R4=R5 (z=1 same)
			jnz		CheckRXBuffer_NoRXOverflow
;
			mov		#RXBuffer,	R4		; reset the pointer
;
CheckRXBuffer_NoRXOverflow
			mov		R4,			&RX_Tail	; save the new pointer
;
CheckRXBuffer_Exit
			eint
			ret
;
;---------------------------------------;---------------------------------------
;   Is the TXBuffer Full?
;
;	R4	-	new head pointer
;	R5	-	temp
;
;---------------------------------------;---------------------------------------
;
CheckForTXBufferFull
			bis		#1<<TXBufferFull,	&Flags	; start with it full
			mov		&TX_Head,	R4
			inc		R4					; R4=TXHead+1
			mov		#TXBuffer+BufferLength,	R5	; R5=Buffer Overflow
			cmp		R4,			R5		; R4=R5 (z=1 same)
			jnz		CheckForTXBufferFull_NoOverflow
;
			mov		#TXBuffer,	R4		; reset the new Head pointer
;
CheckForTXBufferFull_NoOverflow
			cmp		&TX_Tail,	R4		; TXBuffer full?
			jnz		CheckForTXBufferFull_Room	; no, go clear the flag
;
CheckForTXBufferFull_NoRoom
            bis.b   #UCA0TXIE,	&IE2    ; TX IE Enabled
			jmp		CheckForTXBufferFull_Exit	; yes, forget about the character
;
CheckForTXBufferFull_Room
			bic		#1<<TXBufferFull,	&Flags	; not full
;
CheckForTXBufferFull_Exit
			ret
;
;---------------------------------------;---------------------------------------
;   Add a Characters to the TXBuffer
;
;	TXCharacterIn - character to add
;	R4	-	new head pointer
;	R5	-	head pointer
;
;	call	-	CheckForTXBufferFull (R4, R5)
;
;---------------------------------------;---------------------------------------
;
AddToTXBuffer
			dint
			call	#CheckForTXBufferFull
;
			bit		#1<<TXBufferFull,	&Flags	; is the TXBuffer full? (z=0 full)
			jnz		AddToTXBuffer_Exit	; yes, keep trying
;
			mov		&TX_Head,	R5		; no, get the TX_Head pointer
			mov.b	&TXCharacterIn,	0(R5)	; save the character into the TXBuffer
			mov		R4,		&TX_Head	; save the new Head pointer
            bis.b   #UCA0TXIE,	&IE2    ; TX IE Enabled
;
AddToTXBuffer_Exit
			eint
			ret
;
;---------------------------------------;---------------------------------------
;     LCD_Init
;---------------------------------------;---------------------------------------
;
LCD_Init
; delay>20mS done previously
;
			mov.b	#030h,		&DataOut
			call	#LCD_W_Control4Bit
;
			mov		#0FE00h,		R4	; ~200uS
;
LCD_Init_Loop1							; >160uS
			inc	R4
			jnz		LCD_Init_Loop1
;
			mov.b	#030h,		&DataOut
			call	#LCD_W_Control4Bit
;
			mov		#0FE00h,		R4	; ~200uS
;
LCD_Init_Loop2							; >160uS
			inc.b	R4
			jnz		LCD_Init_Loop2
;
			mov.b	#030h,		&DataOut
			call	#LCD_W_Control4Bit
;
			mov		#0FE00h,		R4	; ~200uS
;
LCD_Init_Loop3							; >160uS
			inc.b	R4
			jnz		LCD_Init_Loop3
;
			mov.b	#020h,		&DataOut	; request 4-bit mode
			call	#LCD_W_Control4Bit
;
LCD_Init_Loop4							; >160uS
			inc.b	R4
			jnz		LCD_Init_Loop4
;
			mov.b	#028h,		&DataOut	; 2-line
			call	#LCD_W_Control
;
			mov.b	#0Ch,		&DataOut	; display ON, cursor OFF, no blink
			call	#LCD_W_Control
;
			mov.b	#01h,		&DataOut	; cls
			call	#LCD_W_Control
;
			ret
;
;---------------------------------------;---------------------------------------
;     LCD_R_Control
;---------------------------------------;---------------------------------------
;
LCD_R_Control
			bic.b	#0F0h,		&P1DIR	; data bus IN
;
			bic.b	#1<<RS,		&P2OUT	; select Control register
			bis.b	#1<<R_W,	&P2OUT	; select Read
			bis.b	#1<<E,		&P2OUT	; Strobe high
			nop
;
			mov.b	&P1IN,		R4		; Get P1
			bic.b	#0Fh,		R4		; just upper nibble
			mov.b	R4,			&DataIn	; Save upper nibble
			nop
			bic.b	#1<<E,		&P2OUT	; Strobe low
			nop
;
			bis.b	#1<<E,		&P2OUT	; Strobe high
			nop
;
			mov.b	&P1IN,		R4		; Get P1
			rra.b	R4
			rra.b	R4
			rra.b	R4
			rra.b	R4
			bic.b	#0F0h,		R4		; just upper nibble
			add.b	R4,			&DataIn	; Save upper nibble
			nop
			bic.b	#1<<E,		&P2OUT	; Strobe low
;
			ret
;
;---------------------------------------;---------------------------------------
;     LCD_Chk4Busy
;---------------------------------------;---------------------------------------
;
LCD_Chk4Busy
			call	#LCD_R_Control
;
			bit.b	#1<<7,		&DataIn	; Busy?  bit7=1
			jnz		LCD_Chk4Busy		; yes, wait
	;
			ret
;
;---------------------------------------;---------------------------------------
;     LCD_NibbleOut
;---------------------------------------;---------------------------------------
;
LCD_NibbleOut
			bis.b	#0F0h,		&P1DIR	; Data bus out
;

			bis.b	#1<<E,		&P2OUT	; strobe high
			mov.b	&DataOut,	R4		; Data to output
			bic.b	#0Fh,		R4		; just upper nibble
			mov.b	&P1OUT,		R5		; present port output
			bic.b	#0F0h,		R5		; clear upper nibble
			add.b	R4,			R5		; put 'em together
			mov.b	R5,			&P1OUT	; new port value
			nop
			bic.b	#1<<E,		&P2OUT	; strobe low
			ret
;
;---------------------------------------;---------------------------------------
;     LCD_W_Control
;---------------------------------------;---------------------------------------
;
LCD_W_Control
			call	#LCD_Chk4Busy		; LCD busy?
;
LCD_W_Control_NoChk4Busy
			bis.b	#0F0h,		&P1DIR	; Data bus out
;
			bic.b	#1<<RS,		&P2OUT	; select Control register
			nop
			bic.b	#1<<R_W,	&P2OUT	; LCD write
			nop
			call	#LCD_NibbleOut
;
			rla.b	&DataOut
			rla.b	&DataOut
			rla.b	&DataOut
			rla.b	&DataOut
			call	#LCD_NibbleOut
;
			ret
;
;---------------------------------------;---------------------------------------
;     LCD_W_Data
;---------------------------------------;---------------------------------------
;
LCD_W_Data
			call	#LCD_Chk4Busy		; LCD busy?
;
			bis.b	#0F0h,		&P1DIR	; Data bus out
;
			bis.b	#1<<RS,		&P2OUT	; select Data register
			nop
			bic.b	#1<<R_W,	&P2OUT	; LCD write
			nop
			call	#LCD_NibbleOut
;
			rla.b	&DataOut
			rla.b	&DataOut
			rla.b	&DataOut
			rla.b	&DataOut
			call	#LCD_NibbleOut
;
			ret
;
;---------------------------------------;---------------------------------------
;     LCD_W_Control4Bit
;---------------------------------------;---------------------------------------
;
LCD_W_Control4Bit
			bis.b	#0F0h,		&P1DIR	; Data bus out
;
			bic.b	#1<<RS,		&P2OUT	; select Control register
			nop
			bic.b	#1<<R_W,	&P2OUT	; LCD write
;
			call	#LCD_NibbleOut
;
			ret
;
;---------------------------------------;---------------------------------------
;     Table
;---------------------------------------;---------------------------------------
;
Table
	.align	8
	.char	".-",0						; A
	.align	8
	.char	"-...",0					; B
	.align	8
	.char	"-.-.",0					; C
	.align	8
	.char	"-..",0						; D
	.align	8
	.char	".",0						; E
	.align	8
	.char	"..-.",0					; F
	.align	8
	.char	"--.",0						; G
	.align	8
	.char	"....",0					; H
	.align	8
	.char	"..",0						; I
	.align	8
	.char	".---",0					; J
	.align	8
	.char	"-.-",0						; K
	.align	8
	.char	".-..",0					; L
	.align	8
	.char	"--",0						; M
	.align	8
	.char	"-.",0						; N
	.align	8
	.char	"---",0						; O
	.align	8
	.char	".--.",0					; P
	.align	8
	.char	"--.-",0					; Q
	.align	8
	.char	".-.",0						; R
	.align	8
	.char	"...",0						; S
	.align	8
	.char	"-",0						; T
	.align	8
	.char	"..-",0						; U
	.align	8
	.char	"...-",0					; V
	.align	8
	.char	".--",0						; W
	.align	8
	.char	"-..-",0					; X
	.align	8
	.char	"-.--",0					; Y
	.align	8
	.char	"--..",0					; Z
	.align	8
	.char	"-----",0					; 0
	.align	8
	.char	".----",0					; 1
	.align	8
	.char	"..---",0					; 2
	.align	8
	.char	"...--",0		 			; 3
	.align	8
	.char	"....-",0					; 4
	.align	8
	.char	".....",0					; 5
	.align	8
	.char	"-....",0					; 6
	.align	8
	.char	"--...",0					; 7
	.align	8
	.char	"---..",0					; 8
	.align	8
	.char	"----.",0					; 9
Messages
Message_SIGNON1
	.char	"from the bench 272",0
Message_SIGNON2
	.char	"www.imaginethatnow.com",0
Message_Morse
	.char	"Morse...",0
Message_ftb272
	.char	" ftb272 ",0
;
;---------------------------------------;---------------------------------------
;     Transmitter
;
;	TXCharacterOut	-	character from TXBuffer
;	R14	-	TX_Tail pointer
;	R15	-	temp
;
;---------------------------------------;---------------------------------------
;
TX_ISR
			mov		&TX_Head,	R15
			mov		&TX_Tail,	R14		;
			cmp		R15,		R14		; R14=R15 (z=1 same)
			jz		TX_ISR_BufferEmpty  ; jump if same
;
			mov.b	@R14,		&TXCharacterOut
			mov.b	&TXCharacterOut,	&UCA0TXBUF
			inc		R14
			mov		#TXBuffer+BufferLength,	R15	; R15=Buffer Overflow
			cmp		R14,		R15		; R14=R15 (z=1 same)
			jnz		TX_ISR_NoOverflow
;
			mov		#TXBuffer,	R14		; reset the pointer
;
TX_ISR_NoOverflow
			mov		R14,		&TX_Tail	; save the new pointer
			jmp		TX_ISR_Exit			; and leave
;
TX_ISR_BufferEmpty
            bic.b   #UCA0TXIE,	&IE2    ; TX IE Disabled
;
TX_ISR_Exit
            reti
;
;---------------------------------------;---------------------------------------
;     Receiver
;
;	RXCharacterIn	-	receiver character
;	R14	-	RX_Head pointer
;	R15	-	temp
;
;---------------------------------------;---------------------------------------
;
RX_ISR
            bic.b   #UCA0RXIFG,	&IFG2   ; RX IFG Cleared
			mov.b	&UCA0STAT,	R14
			bit.b	#UCRXERR,	R14		; any error (no error z=1)
			mov.b	&UCA0RXBUF,	&RXCharacterIn
			jnz		RX_ISR_Error		; jump if z=0 (error)
;
RX_ISR_CheckOEOnceMore
			bit.b	#UCOE,		&UCA0STAT	; must check again for this
			jz		RX_ISR_NoErrors		; jump if z=1 (no error)
;
RX_ISR_OE
			mov.b	&UCA0RXBUF,	&RXCharacterIn		; another buffer read clears it
;
RX_ISR_Error							; character dumped, just leave
			jmp		RX_ISR_Exit
;
RX_ISR_NoErrors
			mov		&RX_Head,	R14
			mov.b	&RXCharacterIn,	0(R14)	; move the receiver character into the buffer
			mov		&RX_Head,	R14
			inc		R14					; R4=RXHead+1
			mov		#RXBuffer+BufferLength,	R15	; R5=Buffer Overflow
			cmp		R14,		R15		; R4=R5 (z=1 same)
			jnz		RX_ISR_NoOverflow
;
			mov		#RXBuffer,	R14		; reset the pointer
;
RX_ISR_NoOverflow
			cmp		&RX_Tail,	R14		; R4=RXTail (z=1 same)
			jnz		RX_ISR_Room			; jump if z=0 (Room)
;
RX_ISR_NoRoom
			jmp		RX_ISR_Exit			; forget about the character
;
RX_ISR_Room
			mov		R14,		&RX_Head	; save the new Head pointer
;
RX_ISR_Exit
            reti
;
;---------------------------------------;---------------------------------------
;	TimerA0 Interrupt
;
;	20mS per Tick
;---------------------------------------;---------------------------------------
;
TimerA0_CCI_ISR
           reti
;
;---------------------------------------;---------------------------------------
;	TimerA1 Interrupt (version 0.2)
;---------------------------------------;---------------------------------------
;
TimerA1_CCI_ISR
            reti
;---------------------------------------
DoDIT
			mov		&M_Head,	R10
			mov		#MORSEBuffer+10,	R11
			cmp		R11,		R10		; pointer is ? than the end of buffer
			jl		DoDIT_Room
;
DoDITDumpIt
			mov		#MORSEBuffer,	&M_Head
;
DoDIT_Room
			mov.b	#".",		0(R10)	; add to MBuffer
			inc		&M_Head
DoDIT_Exit
			ret
;---------------------------------------
DoDAH
			mov		&M_Head,	R10
			mov		#MORSEBuffer+10,	R11
			cmp		R11,		R10		; pointer is ? than the end of buffer
			jl		DoDAH_Room
;
DoDAHDumpIt
			mov		#MORSEBuffer,	&M_Head
;
DoDAH_Room
			mov.b	#"-",		0(R10)	; add to MBuffer
			inc		&M_Head
DoDAH_Exit
			ret
;---------------------------------------
DoEndOfLetter
			mov		&M_Head,	R10
			mov		#MORSEBuffer,	R11
			cmp		R10,		R11
			jz		DoEndOfLetter_Exit
;
			mov.b	#0,			0(R10)	; add a 0 to end of MORSE string
			clr		&SymbolCount
			mov		&SymbolCount,	R12
;			rla		R12					; don't need these as 0*8=0
;			rla		R12
;			rla		R12
;
DoEndOfLetter_TableLoop
			mov		#Table,		R11
			add		R12,		R11		; table offset
			mov		#MORSEBuffer,	R10
;
DoEndOfLetter_SymbolLoop
			mov.b	@R10,		R13
			tst.b	R13 				; MBuffer character=0?
			jz		DoEndOfLetter_Maybe	; yes, end of MBuffer go see if Table matches
;
			cmp.b	R13,		0(R11)	; no, compare table to MBuffer
			jnz		DoEndOfLetter_NotFound
;
			inc		R10
			inc		R11
			jmp		DoEndOfLetter_SymbolLoop

;
DoEndOfLetter_Maybe
			mov.b	@R11,		R13
			tst.b	R13				; table character=0?
			jnz		DoEndOfLetter_NotFound
;
DoEndOfLetter_Found
			mov.b	&SymbolCount,	R10
			mov.b	#26,		R11
			cmp.b	R11,		R10		; SymbolCount is ? than 26
			jge		DoEndOfLetter_FoundNumeric
;
DoEndOfLetter_FoundAlpha
			mov.b	#41h,		R11		; Alpha ASCII offset
			add.b	R11,		R10
			mov.b	R10,		&TXCharacterIn
			mov.b	R10,		&LCDCharacterIn
			jmp		DoEndOfLetter_Send
;
DoEndOfLetter_FoundNumeric
			sub.b	R11,		R10
			mov.b	#30h,		R11		; Numeric ASCII offset
			add.b	R11,		R10
			mov.b	R10,		&TXCharacterIn
			mov.b	R10,		&LCDCharacterIn
			jmp		DoEndOfLetter_Send
;
DoEndOfLetter_NotFound
			inc.b	&SymbolCount
			mov.b	&SymbolCount,	R12
			mov.b	#37,		R13
			cmp.b	R12,		R13
			jz		DoEndOfLetter_Done
;
DoEndOfLetter_More
			rla		R12
			rla		R12
			rla		R12
			jmp		DoEndOfLetter_TableLoop
;
DoEndOfLetter_Done
			mov.b	#"?",		&TXCharacterIn
			mov.b	#"?",		&LCDCharacterIn
			jmp		DoEndOfLetter_Send
;
DoEndOfLetter_Send
			call	#AddToTXBufferISR
;
			mov		#MORSEBuffer,	&M_Head
			call	#AddToLCD
;
			jmp		DoEndOfLetter_Exit
;
DoEndOfLetter_Exit
			ret
;---------------------------------------
DoEndOfWord
			call	#DoEndOfLetter
;
			mov.b	#" ",		&TXCharacterIn
			mov.b	#" ",		&LCDCharacterIn
			call	#AddToTXBufferISR
;
			call	#AddToLCD
;
DoEndOfWord_Exit
			ret
;
;---------------------------------------;---------------------------------------
;
;	Timer 0 overflow					; used to Send MORSE
;										; clock=ACLK/8 (~1.6uS clock)
;											ex. DITTime=40, duration=40*1.6mS=64mS
;
;---------------------------------------;---------------------------------------
;
TimerA0_TAI_ISR
			add 	&TA0IV,		PC 		; Add offset to Jump table
			reti 						; Vector 0: No interrupt
			reti 						; Vector 2: TACCR1
			reti 						; Vector 4: TACCR2
			reti 						; Vector 6: Reserved
			reti 						; Vector 8: Reserved
			jmp		TimerA0_TAIF_ISR	; Vector 10: TAIFG interrupt
			reti						; Vector 12: Reserved
			reti 						; Vector 14: Reserved
;
TimerA0_TAIF_ISR
			xor		#1<<DitCLK,	&P2OUT	; toggle the output
			bit		#1<<Pause,	&Flags	; Pause In Process?
			jz		TimerA0_TAIF_ISR_Pause	; no, stop the sound
;
			bic		#MC0,		&TA0CTL	; stop timer0
			bic		#1<<Sound,	&Flags	; clear the sound flag
			bic		#1<<Pause,	&Flags	; clear the pause flag
			jmp		TimerA0_TAIF_ISR_Exit
;
TimerA0_TAIF_ISR_Pause
			bis		#1<<Pause,	&Flags	; now Pause In Process
			bic		#1<<LED1,	&P1OUT	; turn off the sound
			bis		#TACLR,		&TA0CTL	; zero's Timer 0
			mov		&TDIT,		&TA0CCR0	; pause is 1 DIT Time
			jmp		TimerA0_TAIF_ISR_Exit
;
TimerA0_TAIF_ISR_Exit
			reti
;
;---------------------------------------;---------------------------------------
;
;	Timer 1 overflow					; used to Receive MORSE (~83uS clock)
;
;---------------------------------------;---------------------------------------
;
TimerA1_TAI_ISR
			add 	&TA1IV,		PC 		; Add offset to Jump table
			reti 						; Vector 0: No interrupt
			reti 						; Vector 2: TACCR1
			reti 						; Vector 4: TACCR2
			reti 						; Vector 6: Reserved
			reti 						; Vector 8: Reserved
			jmp		TimerA1_TAIF_ISR	; Vector 10: TAIFG interrupt
			reti						; Vector 12: Reserved
			reti 						; Vector 14: Reserved
;
TimerA1_TAIF_ISR
			bis		#1<<T1OVF,	&Flags	; set the flag
			bic		#MC0,		&TA1CTL	; stop timer1
			call	#DoEndOfWord
;
			reti
;
;---------------------------------------;---------------------------------------
;
;	Port 2 change of state
;
;---------------------------------------;---------------------------------------
;
P2_ISR
			bis		#MC0,		&TA1CTL	; start timer1
			bic		#1<<Mike,	&P2IFG	; clear the interrupt
			xor		#1<<Mike,	&P2IES	; next time use opposite edge
			bit		#1<<Mike,	&P2IES	; this is now opposite of this interrupt edge
			jz		P2_ISR_Falling		; was falling edge
;
P2_ISR_Rising							; was falling (end of symbol)
			bit		#1<<T1OVF,	&Flags	; did the timer overflow?
			jnz		P2_ISR_T1OVF		; yes, go on
;
P2_ISR_Rising_OK						; ON Time
			mov		&TA1R,	&OFFTime
			bis		#TACLR,		&TA1CTL	; clear timer1
			mov		&OFFTime,	R10
			mov		&RDIT,		R11
			cmp		R11,		R10		; OFFTime < RDIT?
			jl		P2_ISR_Rising_OFFTimeLTRDIT	; yes
;
			rla		R11					; RDIT*2
			cmp		R11,		R10		; OFFTime < 2*RDIT?
			jl		P2_ISR_Rising_OFFTimeLT2RDIT	; yes
;
			rla		R11					; 2RDIT*2
			add		&RDIT,		R11		; 4RDIT+1
			cmp		R11,		R10		; OFFTime < 5*RDIT?
			jl		P2_ISR_Rising_OFFTimeLT5RDIT	; yes
;
P2_ISR_Rising_OFFTimeGT5RDIT
			call	#DoEndOfWord
;
			jmp		P2_ISR_Exit
;
P2_ISR_Rising_OFFTimeLT5RDIT
			call	#DoEndOfLetter
;
			jmp		P2_ISR_Exit
;
P2_ISR_Rising_OFFTimeLT2RDIT
			nop
			jmp		P2_ISR_Exit
;
P2_ISR_Rising_OFFTimeLTRDIT
			nop
			jmp		P2_ISR_Exit
;
P2_ISR_Falling							; (end of silence)
			bit		#1<<T1OVF,	&Flags	; did the timer overflow?
			jnz		P2_ISR_T1OVF		; yes, go on
;
P2_ISR_Falling_OK						; OFF Time
			mov		&TA1R,	&ONTime
			bis		#TACLR,		&TA1CTL	; clear timer1
			mov		&ONTime,	R10
			mov		&RDIT,		R11
			cmp		R11,		R10		; ONTime < RDIT?
			jl		P2_ISR_Falling_ONTimeLTRDIT	; yes
;
			rla		R11					; RDIT*2
			cmp		R11,		R10		; ONTime < 2*RDIT?
			jl		P2_ISR_Falling_ONTimeLT2RDIT	; yes
;
			rla		R11					; 2RDIT*2
			cmp		R11,		R10		; ONTime < 4*RDIT?
			jl		P2_ISR_Falling_ONTimeLT4RDIT	; yes
;
P2_ISR_Falling_ONTimeGT4RDIT
			mov 	R10,		R11
			add		R11,		R10
			add		R11,		R10		; R10=ONTime*3
			rra		R10
			rra		R10
			rra		R10					; R10=R10/8 (~ONTime/3)
			mov		R10,		&RDIT
			call	#DoDAH
;
			jmp		P2_ISR_Exit
;
P2_ISR_Falling_ONTimeLT4RDIT
			call	#DoDAH
;
			jmp		P2_ISR_Exit
;
P2_ISR_Falling_ONTimeLT2RDIT
			call	#DoDIT
;
			jmp		P2_ISR_Exit
;
P2_ISR_Falling_ONTimeLTRDIT
			mov		&ONTime,	&RDIT
			call	#DoDIT
;
			jmp		P2_ISR_Exit
;
P2_ISR_T1OVF
			bis		#TACLR,		&TA1CTL	; clear timer1
			bic		#1<<T1OVF,	&Flags	; clear the overflow
;
P2_ISR_Exit
            reti                       	;
;
;---------------------------------------;---------------------------------------
;     Other Interrupts just Return
;---------------------------------------;---------------------------------------
;
other_ISR
            reti                       	;
;
;---------------------------------------;---------------------------------------
;     Add to TXBuffer
;---------------------------------------;---------------------------------------
;
AddToTXBufferISR
			push	R4
			push	R5
			call	#CheckForTXBufferFull
;
			bit		#1<<TXBufferFull,	&Flags	; is the TXBuffer full? (z=0 full)
			jnz		AddToTXBufferISR_Exit	; yes, forget it
;
			mov		&TX_Head,	R5		; no, get the TX_Head pointer
			mov.b	&TXCharacterIn,	0(R5)	; save the character into the TXBuffer
			mov		R4,		&TX_Head	; save the new Head pointer
           	bis.b   #UCA0TXIE,	&IE2    ; TX IE Enabled
;
AddToTXBufferISR_Exit
			pop		R5
			pop		R4
			ret
;
;---------------------------------------;---------------------------------------
;     Add to LCD
;---------------------------------------;---------------------------------------
;
AddToLCD
			push	R4
			push	R5
			push 	R6
			mov		&Line1Pointer,	R4
			mov		&Line1Pointer,	R5
			inc		R5
			mov.b	#7,			R6
;
AddToLCD_Loop
			mov.b	@R5+,		0(R4)
			inc		R4
			dec.b	R6
			tst.b	R6
			jnz		AddToLCD_Loop
;
			mov.b	&LCDCharacterIn,	0(R4)
			call	#SendToLCDLine1
;
AddToLCD_Exit
			pop		R6
			pop		R5
			pop		R4
			ret
;
;---------------------------------------;---------------------------------------
;           Interrupt Vectors
;---------------------------------------;---------------------------------------
;
            .sect   ".reset"            ; MSP430 RESET Vector
            .short  _main
            .sect   ".int00"
            .short  other_ISR
            .sect	".int01"			;
            .short	other_ISR
            .sect   ".int02"            ; PORT1
            .short  other_ISR
            .sect	".int03"
            .short	P2_ISR				; PORT2
            .sect	".int04"
            .short	other_ISR			;
            .sect	".int05"
            .short	other_ISR			; ADC10
            .sect	".int06"
            .short	TX_ISR				; USCIAB0TX
            .sect	".int07"
            .short	RX_ISR				; USCIAB0RX
            .sect	".int08"
            .short	TimerA0_TAI_ISR		; TIMER0_A0_TAI
            .sect	".int09"
            .short	TimerA0_CCI_ISR		; TIMER0_A0_CCI
            .sect	".int10"
            .short	other_ISR			; WDT
            .sect	".int11"
            .short	other_ISR			; COMPARATORA
            .sect	".int12"
            .short	TimerA1_TAI_ISR 	; TIMER0_A1_TAI
            .sect	".int13"
            .short	TimerA1_CCI_ISR		; TIMER0_A1_CCI
            .sect	".int14"
            .short	other_ISR			; NMI

            .end
