/*////////////////////////////////////////////////////////////////////////////
	PROJECT:	PIC Ethernet
	COPYRIGHT	 2009 Dean J. Boman

	FILE:		TC77_read.h
	AUTHOR:		Dean J. Boman

	OVERVIEW:
		This implements the SPI interface to communicate with the TC77 temperature
		sensor

	REVISION HISTORY:

	Dec. 24, 2011	Initial development



*/////////////////////////////////////////////////////////////////////////////
#include "Compiler.h"
#include "Configs/HWP BOMAN.h"
#include "TC77_read.h"
#include "typedef.h"
#include "GenericTypeDefs.h"		//Needed for TRUE/FALSE
#include <delays.h>


/**
	This routine reads the temperature from the specified sensor.  If the read
	was successful the temperature is returned as a float in degrees F.  This routine 
	takes approximatly 8 msec to complete.

	@param uint8 - Address 0=local sensor, 1=remote sensor

  	@return float - Temperature (degrees F)

*/

float TempRead(uint8 Address)
{

	uint8 BitCounter;
	uint8 InputBit;
	int16 TempData;
	int16 TempDataSigned;
	float Temperature;

	TempData = 0;

	//Set the data port for input
	TSENSE_SPI_DATA_TRIS = 1;

	//Set the clock to the low state 
	CLOCK_LOW;
	Delay100TCYx(CLOCKDELAY);

	if(Address == 0)
		TSENSE_SPI_LOCAL_CS = 0;
	else
		TSENSE_SPI_REMOTE_CS = 0;	
		
	//Read 16 bits of temperature data
	for (BitCounter = 0;BitCounter < 16;BitCounter++)
	{
		Delay100TCYx(CLOCKDELAY);

		InputBit = TSENSE_SPI_DATA;

		TempData = (TempData << 1) | (InputBit & 0x01);

		CLOCK_HIGH;

		Delay100TCYx(CLOCKDELAY);

		CLOCK_LOW;		//data is output from the sensor on the falling edge

		//Add additional delay after 8 clocks
		if(BitCounter == 7)
			Delay100TCYx(3 * CLOCKDELAY);
	}

	
	//Set the data port for output and set the data bit low
	TSENSE_SPI_DATA_TRIS = 0;
	TSENSE_SPI_DATA = 0;

	Delay100TCYx(3 * CLOCKDELAY);
/*   The following is not needed so remove to minimize traffic

	//Send all zeros to the sensor to put it in the continous conversion mode
	for (BitCounter = 0;BitCounter < 16;BitCounter++)
	{
		Delay100TCYx(CLOCKDELAY);

		CLOCK_HIGH;

		Delay100TCYx(CLOCKDELAY);

		CLOCK_LOW;

		//Add additional delay after 8 clocks
		if(BitCounter == 7)
			Delay100TCYx(3 * CLOCKDELAY);
	}
*/
	Delay100TCYx(CLOCKDELAY);

	//Set the CS high
	TSENSE_SPI_LOCAL_CS = 1;
	TSENSE_SPI_REMOTE_CS = 1;

	//Set the data line for input
	TSENSE_SPI_DATA_TRIS = 1;

	//Shift the data down to remove the lower 5 bits (sign extening the upper bits)
	//Note:  The right shift doesn't copy the sign bit so we use a /32 instead of a >>5
	TempDataSigned = (TempData / 32);

	//If the wire to the sensor breaks all ones are returned so we set the temp high which will
	//force a circulate condition.
	if(TempData == 0xffff) TempDataSigned = 1929;		//All ones indicate invalid reading so set temp to 900

	//Convert the temperature to units of degree and make converstion from C to F
	Temperature = ((0.25 * TempDataSigned)* 9/5) + 32.0;

	return(Temperature);
}



