/*////////////////////////////////////////////////////////////////////////////
    PROJECT:    System Manager
    COPYRIGHT    2011 Dean J. Boman

    FILE:       Time_conversion.c
    AUTHOR:     Dean J. Boman

    OVERVIEW:
        This routine converts the output of the SNTP function (seconds since Jan 1, 1970) to real time
        with a specific UTC offset.
        
        This code is an adaptation of code found on the Microchip forum dated Nov. 14, 2007 under the title 
        "Time in web page" under the Ethernet heading.  For the current implementation the Time structure was
        modifed to conform to the existing code and a time zone correction factor was added.



    REVISION HISTORY:

	Dec. 15. 2011	Initial adaptation





*/////////////////////////////////////////////////////////////////////////////
#include "Time_conversion.h"
	
void BinaryToDate(unsigned long binary,TIME *datetime) {
 
     unsigned long hour;
     unsigned long day;
     unsigned long minute;
     unsigned long second;
     unsigned long month;
     unsigned long year;
     
     unsigned long whole_minutes;
     unsigned long whole_hours;
     unsigned long whole_days;
     unsigned long whole_days_since_1968;
     unsigned long leap_year_periods;
     unsigned long days_since_current_lyear;
     unsigned long whole_years;
     unsigned long days_since_first_of_year;
     unsigned long days_to_month;
     unsigned long day_of_week;
     
     unsigned long DaysToMonth[13] = {
 	    0,31,59,90,120,151,181,212,243,273,304,334,365
 	};
        
     whole_minutes = binary / 60;
     second = binary - (60 * whole_minutes);                 // leftover seconds
     
     whole_hours  = whole_minutes / 60;
     minute = whole_minutes - (60 * whole_hours);            // leftover minutes
     
     whole_days   = whole_hours / 24;
     hour         = whole_hours - (24 * whole_days);         // leftover hours
         
     whole_days_since_1968 = whole_days + 365 + 366;
     leap_year_periods = whole_days_since_1968 / ((4 * 365) + 1);
     
     days_since_current_lyear = whole_days_since_1968 % ((4 * 365) + 1);
         
     // if days are after a current leap year then add a leap year period
     if ((days_since_current_lyear >= (31 + 29))) {
         leap_year_periods++;
     }
     whole_years = (whole_days_since_1968 - leap_year_periods) / 365;
     days_since_first_of_year = whole_days_since_1968 - (whole_years * 365) - leap_year_periods;
     
     if ((days_since_current_lyear <= 365) && (days_since_current_lyear >= 60)) {
         days_since_first_of_year++;
     }
     year = whole_years + 68;        
     
     // setup for a search for what month it is based on how many days have past
     //   within the current year
     month = 13;
     days_to_month = 366;
     while (days_since_first_of_year < days_to_month) {
         month--;
         days_to_month = DaysToMonth[month];
         if ((month >= 2) && ((year % 4) == 0))
         {
             days_to_month++;
         }
     }
     day = days_since_first_of_year - days_to_month + 1;
     
     day_of_week = (whole_days  + 4) % 7;
     
     
	//Adjust for the time zone offset
	if(hour < GMT_OFFSET)
		hour += (24 - GMT_OFFSET);
	else
		hour -= GMT_OFFSET;

	//Handle the case where the GMT day has advanced but local time has not
	//This simple approach doesn't make corrections to the month or year
	if(hour > (24 - GMT_OFFSET))
		day -= 1;
         
     month += 1;		//Convert to a 1 to 12 based month
     year -= 100;		//Set the base year to 2000
         
     
     datetime->DayOfYear = days_since_first_of_year;      /* days since January 1 - [0,365]       */
     datetime->Second  = second;        /* seconds after the minute - [0,59]    */
     datetime->Minute  = minute;        /* minutes after the hour - [0,59]      */
     datetime->Hour = hour;          /* hours since midnight - [0,23]        */
     datetime->Day = day;           /* day of the month - [1,31]            */
     datetime->WeekDay = day_of_week;   /* days since Sunday - [0,6]            */
     datetime->Month  = month;         /* months since January - [1,12]        */
     datetime->Year = year;          /* years since 2000                     */
     
     
     
 } 

